"C:\Program Files\Java\jdk1.5.0_11\/bin/java" -Dswing.defaultlaf=net.beeger.squareness.SquarenessLookAndFeel -cp ../drop/laf-widget-tst.jar;../drop/laf-widget.jar;augmented/squareness.jar test.ButtonRolloverDemo -highcontrast