set JAVA="C:\Program Files\Java\jdk1.6.0_10\bin\java"

%JAVA% -cp lib/swingx.jar;lib/swing-worker.jar;../substance/lib/forms-1.1.0.jar;../substance/lib/test/fest-swing-1.2a3.jar;../substance/lib/test/fest-reflect-1.1.jar;../substance/lib/test/fest-util-1.1.jar;../substance/lib/test/fest-assert-1.1.jar;drop/substance-swingx-tst.jar;drop/substance-swingx.jar;../substance/drop/substance.jar;../substance/drop/substance-tst.jar docrobot.StatusBarRunner
%JAVA% -cp lib/swingx.jar;lib/swing-worker.jar;../substance/lib/forms-1.1.0.jar;../substance/lib/test/fest-swing-1.2a3.jar;../substance/lib/test/fest-reflect-1.1.jar;../substance/lib/test/fest-util-1.1.jar;../substance/lib/test/fest-assert-1.1.jar;drop/substance-swingx-tst.jar;drop/substance-swingx.jar;../substance/drop/substance.jar;../substance/drop/substance-tst.jar docrobot.HeaderRunner
%JAVA% -cp lib/swingx.jar;lib/swing-worker.jar;../substance/lib/forms-1.1.0.jar;../substance/lib/test/fest-swing-1.2a3.jar;../substance/lib/test/fest-reflect-1.1.jar;../substance/lib/test/fest-util-1.1.jar;../substance/lib/test/fest-assert-1.1.jar;drop/substance-swingx-tst.jar;drop/substance-swingx.jar;../substance/drop/substance.jar;../substance/drop/substance-tst.jar docrobot.LoginDialogRunner
%JAVA% -cp lib/swingx.jar;lib/swing-worker.jar;../substance/lib/forms-1.1.0.jar;../substance/lib/test/fest-swing-1.2a3.jar;../substance/lib/test/fest-reflect-1.1.jar;../substance/lib/test/fest-util-1.1.jar;../substance/lib/test/fest-assert-1.1.jar;drop/substance-swingx-tst.jar;drop/substance-swingx.jar;../substance/drop/substance.jar;../substance/drop/substance-tst.jar docrobot.MonthViewRunner
%JAVA% -cp lib/swingx.jar;lib/swing-worker.jar;../substance/lib/forms-1.1.0.jar;../substance/lib/test/fest-swing-1.2a3.jar;../substance/lib/test/fest-reflect-1.1.jar;../substance/lib/test/fest-util-1.1.jar;../substance/lib/test/fest-assert-1.1.jar;drop/substance-swingx-tst.jar;drop/substance-swingx.jar;../substance/drop/substance.jar;../substance/drop/substance-tst.jar docrobot.TaskPaneRunner
%JAVA% -cp lib/swingx.jar;lib/swing-worker.jar;../substance/lib/forms-1.1.0.jar;../substance/lib/test/fest-swing-1.2a3.jar;../substance/lib/test/fest-reflect-1.1.jar;../substance/lib/test/fest-util-1.1.jar;../substance/lib/test/fest-assert-1.1.jar;drop/substance-swingx-tst.jar;drop/substance-swingx.jar;../substance/drop/substance.jar;../substance/drop/substance-tst.jar docrobot.TitledPanelRunner
%JAVA% -cp lib/swingx.jar;lib/swing-worker.jar;../substance/lib/forms-1.1.0.jar;../substance/lib/test/fest-swing-1.2a3.jar;../substance/lib/test/fest-reflect-1.1.jar;../substance/lib/test/fest-util-1.1.jar;../substance/lib/test/fest-assert-1.1.jar;drop/substance-swingx-tst.jar;drop/substance-swingx.jar;../substance/drop/substance.jar;../substance/drop/substance-tst.jar docrobot.ErrorPaneRunner
