"C:\Program Files\Java\jdk1.5.0_11\/bin/java" -Dswing.defaultlaf=com.pagosoft.plaf.PgsLookAndFeel -cp ../drop/laf-widget-tst.jar;../drop/laf-widget.jar;augmented/PgsLookAndFeel.jar test.ButtonRolloverDemo -highcontrast