/*
 * Copyright (c) 2005-2010 Substance Kirill Grouchnikov. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  o Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer. 
 *     
 *  o Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution. 
 *     
 *  o Neither the name of Substance Kirill Grouchnikov nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission. 
 *     
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package org.pushingpixels.substance.internal.animation;

import java.awt.Component;

import org.pushingpixels.lafwidget.animation.AnimationConfigurationManager;
import org.pushingpixels.trident.Timeline;
import org.pushingpixels.trident.Timeline.RepeatBehavior;
import org.pushingpixels.trident.Timeline.TimelineState;
import org.pushingpixels.trident.swing.SwingRepaintCallback;

public class IconGlowTracker {
	private Timeline iconGlowTimeline;

	private Component component;

	public IconGlowTracker(Component component) {
		this.component = component;

		this.iconGlowTimeline = new Timeline(this.component);
		AnimationConfigurationManager.getInstance().configureTimeline(
				this.iconGlowTimeline);
		this.iconGlowTimeline.setDuration(10 * this.iconGlowTimeline
				.getDuration());
		this.iconGlowTimeline.addCallback(new SwingRepaintCallback(component));
		this.iconGlowTimeline.setName("Icon glow");
	}

	public void play() {
		this.iconGlowTimeline.playLoop(RepeatBehavior.REVERSE);
	}

	public void play(int fullLoopCount) {
		this.iconGlowTimeline.playLoop(fullLoopCount, RepeatBehavior.REVERSE);
	}

	public boolean isPlaying() {
		return (this.iconGlowTimeline.getState() != TimelineState.IDLE);
	}

	public void cancel() {
		if (this.iconGlowTimeline.getState() != TimelineState.IDLE) {
			this.iconGlowTimeline.cancelAtCycleBreak();
		}
	}

	public float getIconGlowPosition() {
		return this.iconGlowTimeline.getTimelinePosition();
	}
}
