alias JAVA="/System/Library/Frameworks/JavaVM.framework/Versions/1.6.0/Home/bin/java"

JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.Autumn
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.Business
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.BusinessBlackSteel
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.BusinessBlueSteel
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.ChallengerDeep
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.CeruleanSkin
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.Creme
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.CremeCoffee
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.Dust
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.DustCoffee
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.EmeraldDusk
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.Gemini
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.Graphite
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.GraphiteGlass
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.GraphiteAqua
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.Magellan
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.Mariner
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.MistSilver
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.MistAqua
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.Moderate
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.Nebula
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.NebulaBrickWall
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.Raven
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.Sahara
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.Twilight
