"C:\Program Files\Java\jdk1.5.0_11\/bin/java" -Dswing.defaultlaf=com.birosoft.liquid.LiquidLookAndFeel -cp ../drop/laf-widget-tst.jar;../drop/laf-widget.jar;augmented/liquidlnf.jar test.check.SampleFrame