set JAVA="/System/Library/Frameworks/JavaVM.framework/Versions/1.6.0/Home/bin/java"

JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.watermarks.CrosshatchWatermark
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.watermarks.NullWatermark
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.watermarks.StripesWatermark

JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.watermarks.ImageWatermarkAngelina
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.watermarks.ImageWatermarkBeyonce
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.watermarks.ImageWatermarkDominic
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.watermarks.ImageWatermarkTerry
