set JAVA="/System/Library/Frameworks/JavaVM.framework/Versions/1.6.0/Home/bin/java"

JAVA -cp ../substance/drop/substance-flamingo-tools.jar:../substance/lib/build/forms-1.2.0.jar:../flamingo/drop/flamingo.jar:../flamingo/drop/flamingo-tst.jar:../trident/drop/trident.jar:../substance/drop/substance.jar:./drop/substance-flamingo.jar:./drop/substance-flamingo-tst.jar:../substance/lib/test/fest-swing-1.2.jar:../substance/lib/test/fest-reflect-1.2.jar:../substance/lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.Autumn
JAVA -cp ../substance/drop/substance-flamingo-tools.jar:../substance/lib/build/forms-1.2.0.jar:../flamingo/drop/flamingo.jar:../flamingo/drop/flamingo-tst.jar:../trident/drop/trident.jar:../substance/drop/substance.jar:./drop/substance-flamingo.jar:./drop/substance-flamingo-tst.jar:../substance/lib/test/fest-swing-1.2.jar:../substance/lib/test/fest-reflect-1.2.jar:../substance/lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.Business
JAVA -cp ../substance/drop/substance-flamingo-tools.jar:../substance/lib/build/forms-1.2.0.jar:../flamingo/drop/flamingo.jar:../flamingo/drop/flamingo-tst.jar:../trident/drop/trident.jar:../substance/drop/substance.jar:./drop/substance-flamingo.jar:./drop/substance-flamingo-tst.jar:../substance/lib/test/fest-swing-1.2.jar:../substance/lib/test/fest-reflect-1.2.jar:../substance/lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.BusinessBlackSteel
JAVA -cp ../substance/drop/substance-flamingo-tools.jar:../substance/lib/build/forms-1.2.0.jar:../flamingo/drop/flamingo.jar:../flamingo/drop/flamingo-tst.jar:../trident/drop/trident.jar:../substance/drop/substance.jar:./drop/substance-flamingo.jar:./drop/substance-flamingo-tst.jar:../substance/lib/test/fest-swing-1.2.jar:../substance/lib/test/fest-reflect-1.2.jar:../substance/lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.BusinessBlueSteel
JAVA -cp ../substance/drop/substance-flamingo-tools.jar:../substance/lib/build/forms-1.2.0.jar:../flamingo/drop/flamingo.jar:../flamingo/drop/flamingo-tst.jar:../trident/drop/trident.jar:../substance/drop/substance.jar:./drop/substance-flamingo.jar:./drop/substance-flamingo-tst.jar:../substance/lib/test/fest-swing-1.2.jar:../substance/lib/test/fest-reflect-1.2.jar:../substance/lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.ChallengerDeep
JAVA -cp ../substance/drop/substance-flamingo-tools.jar:../substance/lib/build/forms-1.2.0.jar:../flamingo/drop/flamingo.jar:../flamingo/drop/flamingo-tst.jar:../trident/drop/trident.jar:../substance/drop/substance.jar:./drop/substance-flamingo.jar:./drop/substance-flamingo-tst.jar:../substance/lib/test/fest-swing-1.2.jar:../substance/lib/test/fest-reflect-1.2.jar:../substance/lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.CeruleanSkin
JAVA -cp ../substance/drop/substance-flamingo-tools.jar:../substance/lib/build/forms-1.2.0.jar:../flamingo/drop/flamingo.jar:../flamingo/drop/flamingo-tst.jar:../trident/drop/trident.jar:../substance/drop/substance.jar:./drop/substance-flamingo.jar:./drop/substance-flamingo-tst.jar:../substance/lib/test/fest-swing-1.2.jar:../substance/lib/test/fest-reflect-1.2.jar:../substance/lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.Creme
JAVA -cp ../substance/drop/substance-flamingo-tools.jar:../substance/lib/build/forms-1.2.0.jar:../flamingo/drop/flamingo.jar:../flamingo/drop/flamingo-tst.jar:../trident/drop/trident.jar:../substance/drop/substance.jar:./drop/substance-flamingo.jar:./drop/substance-flamingo-tst.jar:../substance/lib/test/fest-swing-1.2.jar:../substance/lib/test/fest-reflect-1.2.jar:../substance/lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.CremeCoffee
JAVA -cp ../substance/drop/substance-flamingo-tools.jar:../substance/lib/build/forms-1.2.0.jar:../flamingo/drop/flamingo.jar:../flamingo/drop/flamingo-tst.jar:../trident/drop/trident.jar:../substance/drop/substance.jar:./drop/substance-flamingo.jar:./drop/substance-flamingo-tst.jar:../substance/lib/test/fest-swing-1.2.jar:../substance/lib/test/fest-reflect-1.2.jar:../substance/lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.Dust
JAVA -cp ../substance/drop/substance-flamingo-tools.jar:../substance/lib/build/forms-1.2.0.jar:../flamingo/drop/flamingo.jar:../flamingo/drop/flamingo-tst.jar:../trident/drop/trident.jar:../substance/drop/substance.jar:./drop/substance-flamingo.jar:./drop/substance-flamingo-tst.jar:../substance/lib/test/fest-swing-1.2.jar:../substance/lib/test/fest-reflect-1.2.jar:../substance/lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.DustCoffee
JAVA -cp ../substance/drop/substance-flamingo-tools.jar:../substance/lib/build/forms-1.2.0.jar:../flamingo/drop/flamingo.jar:../flamingo/drop/flamingo-tst.jar:../trident/drop/trident.jar:../substance/drop/substance.jar:./drop/substance-flamingo.jar:./drop/substance-flamingo-tst.jar:../substance/lib/test/fest-swing-1.2.jar:../substance/lib/test/fest-reflect-1.2.jar:../substance/lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.EmeraldDusk
JAVA -cp ../substance/drop/substance-flamingo-tools.jar:../substance/lib/build/forms-1.2.0.jar:../flamingo/drop/flamingo.jar:../flamingo/drop/flamingo-tst.jar:../trident/drop/trident.jar:../substance/drop/substance.jar:./drop/substance-flamingo.jar:./drop/substance-flamingo-tst.jar:../substance/lib/test/fest-swing-1.2.jar:../substance/lib/test/fest-reflect-1.2.jar:../substance/lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.Gemini
JAVA -cp ../substance/drop/substance-flamingo-tools.jar:../substance/lib/build/forms-1.2.0.jar:../flamingo/drop/flamingo.jar:../flamingo/drop/flamingo-tst.jar:../trident/drop/trident.jar:../substance/drop/substance.jar:./drop/substance-flamingo.jar:./drop/substance-flamingo-tst.jar:../substance/lib/test/fest-swing-1.2.jar:../substance/lib/test/fest-reflect-1.2.jar:../substance/lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.Graphite
JAVA -cp ../substance/drop/substance-flamingo-tools.jar:../substance/lib/build/forms-1.2.0.jar:../flamingo/drop/flamingo.jar:../flamingo/drop/flamingo-tst.jar:../trident/drop/trident.jar:../substance/drop/substance.jar:./drop/substance-flamingo.jar:./drop/substance-flamingo-tst.jar:../substance/lib/test/fest-swing-1.2.jar:../substance/lib/test/fest-reflect-1.2.jar:../substance/lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.GraphiteAqua
JAVA -cp ../substance/drop/substance-flamingo-tools.jar:../substance/lib/build/forms-1.2.0.jar:../flamingo/drop/flamingo.jar:../flamingo/drop/flamingo-tst.jar:../trident/drop/trident.jar:../substance/drop/substance.jar:./drop/substance-flamingo.jar:./drop/substance-flamingo-tst.jar:../substance/lib/test/fest-swing-1.2.jar:../substance/lib/test/fest-reflect-1.2.jar:../substance/lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.GraphiteGlass
JAVA -cp ../substance/drop/substance-flamingo-tools.jar:../substance/lib/build/forms-1.2.0.jar:../flamingo/drop/flamingo.jar:../flamingo/drop/flamingo-tst.jar:../trident/drop/trident.jar:../substance/drop/substance.jar:./drop/substance-flamingo.jar:./drop/substance-flamingo-tst.jar:../substance/lib/test/fest-swing-1.2.jar:../substance/lib/test/fest-reflect-1.2.jar:../substance/lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.Magellan
JAVA -cp ../substance/drop/substance-flamingo-tools.jar:../substance/lib/build/forms-1.2.0.jar:../flamingo/drop/flamingo.jar:../flamingo/drop/flamingo-tst.jar:../trident/drop/trident.jar:../substance/drop/substance.jar:./drop/substance-flamingo.jar:./drop/substance-flamingo-tst.jar:../substance/lib/test/fest-swing-1.2.jar:../substance/lib/test/fest-reflect-1.2.jar:../substance/lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.MistSilver
JAVA -cp ../substance/drop/substance-flamingo-tools.jar:../substance/lib/build/forms-1.2.0.jar:../flamingo/drop/flamingo.jar:../flamingo/drop/flamingo-tst.jar:../trident/drop/trident.jar:../substance/drop/substance.jar:./drop/substance-flamingo.jar:./drop/substance-flamingo-tst.jar:../substance/lib/test/fest-swing-1.2.jar:../substance/lib/test/fest-reflect-1.2.jar:../substance/lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.MistAqua
JAVA -cp ../substance/drop/substance-flamingo-tools.jar:../substance/lib/build/forms-1.2.0.jar:../flamingo/drop/flamingo.jar:../flamingo/drop/flamingo-tst.jar:../trident/drop/trident.jar:../substance/drop/substance.jar:./drop/substance-flamingo.jar:./drop/substance-flamingo-tst.jar:../substance/lib/test/fest-swing-1.2.jar:../substance/lib/test/fest-reflect-1.2.jar:../substance/lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.Moderate
JAVA -cp ../substance/drop/substance-flamingo-tools.jar:../substance/lib/build/forms-1.2.0.jar:../flamingo/drop/flamingo.jar:../flamingo/drop/flamingo-tst.jar:../trident/drop/trident.jar:../substance/drop/substance.jar:./drop/substance-flamingo.jar:./drop/substance-flamingo-tst.jar:../substance/lib/test/fest-swing-1.2.jar:../substance/lib/test/fest-reflect-1.2.jar:../substance/lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.Nebula
JAVA -cp ../substance/drop/substance-flamingo-tools.jar:../substance/lib/build/forms-1.2.0.jar:../flamingo/drop/flamingo.jar:../flamingo/drop/flamingo-tst.jar:../trident/drop/trident.jar:../substance/drop/substance.jar:./drop/substance-flamingo.jar:./drop/substance-flamingo-tst.jar:../substance/lib/test/fest-swing-1.2.jar:../substance/lib/test/fest-reflect-1.2.jar:../substance/lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.NebulaBrickWall
JAVA -cp ../substance/drop/substance-flamingo-tools.jar:../substance/lib/build/forms-1.2.0.jar:../flamingo/drop/flamingo.jar:../flamingo/drop/flamingo-tst.jar:../trident/drop/trident.jar:../substance/drop/substance.jar:./drop/substance-flamingo.jar:./drop/substance-flamingo-tst.jar:../substance/lib/test/fest-swing-1.2.jar:../substance/lib/test/fest-reflect-1.2.jar:../substance/lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.OfficeBlack2007
JAVA -cp ../substance/drop/substance-flamingo-tools.jar:../substance/lib/build/forms-1.2.0.jar:../flamingo/drop/flamingo.jar:../flamingo/drop/flamingo-tst.jar:../trident/drop/trident.jar:../substance/drop/substance.jar:./drop/substance-flamingo.jar:./drop/substance-flamingo-tst.jar:../substance/lib/test/fest-swing-1.2.jar:../substance/lib/test/fest-reflect-1.2.jar:../substance/lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.OfficeBlue2007
JAVA -cp ../substance/drop/substance-flamingo-tools.jar:../substance/lib/build/forms-1.2.0.jar:../flamingo/drop/flamingo.jar:../flamingo/drop/flamingo-tst.jar:../trident/drop/trident.jar:../substance/drop/substance.jar:./drop/substance-flamingo.jar:./drop/substance-flamingo-tst.jar:../substance/lib/test/fest-swing-1.2.jar:../substance/lib/test/fest-reflect-1.2.jar:../substance/lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.OfficeSilver2007
JAVA -cp ../substance/drop/substance-flamingo-tools.jar:../substance/lib/build/forms-1.2.0.jar:../flamingo/drop/flamingo.jar:../flamingo/drop/flamingo-tst.jar:../trident/drop/trident.jar:../substance/drop/substance.jar:./drop/substance-flamingo.jar:./drop/substance-flamingo-tst.jar:../substance/lib/test/fest-swing-1.2.jar:../substance/lib/test/fest-reflect-1.2.jar:../substance/lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.Raven
JAVA -cp ../substance/drop/substance-flamingo-tools.jar:../substance/lib/build/forms-1.2.0.jar:../flamingo/drop/flamingo.jar:../flamingo/drop/flamingo-tst.jar:../trident/drop/trident.jar:../substance/drop/substance.jar:./drop/substance-flamingo.jar:./drop/substance-flamingo-tst.jar:../substance/lib/test/fest-swing-1.2.jar:../substance/lib/test/fest-reflect-1.2.jar:../substance/lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.Sahara
JAVA -cp ../substance/drop/substance-flamingo-tools.jar:../substance/lib/build/forms-1.2.0.jar:../flamingo/drop/flamingo.jar:../flamingo/drop/flamingo-tst.jar:../trident/drop/trident.jar:../substance/drop/substance.jar:./drop/substance-flamingo.jar:./drop/substance-flamingo-tst.jar:../substance/lib/test/fest-swing-1.2.jar:../substance/lib/test/fest-reflect-1.2.jar:../substance/lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.skins.Twilight
