set JAVA="/System/Library/Frameworks/JavaVM.framework/Versions/1.6.0/Home/bin/java"

JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.schemes.AquaScheme
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.schemes.BarbyPinkScheme
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.schemes.BottleGreenScheme
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.schemes.BrownScheme
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.schemes.CharcoalScheme
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.schemes.CremeScheme
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.schemes.DarkVioletScheme
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.schemes.EbonyScheme
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.schemes.JadeForestScheme
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.schemes.LightAquaScheme
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.schemes.LimeGreenScheme
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.schemes.OliveScheme
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.schemes.OrangeScheme
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.schemes.PurpleScheme
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.schemes.RaspberryScheme
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.schemes.SepiaScheme
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.schemes.SteelBlueScheme
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.schemes.SunGlareScheme
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.schemes.SunsetScheme
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.schemes.TerracottaScheme
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.schemes.UltramarineScheme

JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.schemes.DerivedSaturatedScheme
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.schemes.DerivedDesaturatedScheme
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.schemes.DerivedInvertedScheme
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.schemes.DerivedNegatedScheme
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.schemes.DerivedHueShiftedScheme
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.schemes.DerivedShadedScheme
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.schemes.DerivedTintedScheme
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.schemes.DerivedTonedScheme
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.schemes.DerivedShiftedScheme
JAVA -DSubstance.controlFont="Lucida Grande 12" -cp ./drop/substance-tools.jar:./drop/substance.jar:./drop/substance-tst.jar:./lib/forms-1.2.0.jar:./lib/test/fest-swing-1.2.jar:./lib/test/fest-reflect-1.2.jar:./lib/test/fest-util-1.1.2.jar tools.docrobot.RobotMain tools.docrobot.schemes.DerivedShiftedBackgroundScheme
